#version 120

#define AO
#define EmissiveRecolor
#define LightShaft 1 //[0 1 2]
#define Fog
//#define LightmapBanding
//#define RPSupport
#define RPSFormat 0 //[0 1 2]
#define ShadowColor
#define WaterFog
#define WaterRefraction
//#define WorldTimeAnimation
#define AnimationSpeed 1.00 //[0.25 0.50 0.75 1.00 1.25 1.50 1.75 2.00 2.50 3.00 3.50 4.00 5.00 6.00 7.00 8.00]

const int shadowMapResolution = 2048; //[1024 2048 3072 4096]
const float shadowDistance = 256.0; //[128.0 256.0 512.0]
const float shadowMapBias = 1.0-25.6/shadowDistance;

#define MAX_COLOR_RANGE 32.0
#define MAX_COLORB_RANGE 8.0

varying vec3 lightVector;
varying vec3 upVec;
varying vec3 sunVec;
varying vec3 moonVec;
varying float sunVisibility;
varying float moonVisibility;
varying float handRecolor;

varying vec4 texcoord;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex5;
#ifdef RPSupport
uniform sampler2D colortex6;
#endif
uniform sampler2D depthtex0;
#if LightShaft == 2
uniform sampler2D depthtex1;
#endif
uniform sampler2D noisetex;

#if defined RPSupport || LightShaft == 2
uniform sampler2DShadow shadowtex0;
#ifdef ShadowColor
uniform sampler2DShadow shadowtex1;
uniform sampler2DShadow shadowcolor;
#endif
#endif

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;
uniform float rainStrength;
uniform ivec2 eyeBrightnessSmooth;
uniform int isEyeInWater;
uniform int worldTime;
uniform float frameTimeCounter;
uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform float viewWidth;
uniform float viewHeight;
uniform float far;
uniform float near;
uniform float aspectRatio;

float time = float(worldTime);
float tB = max(sin(time/12000*22/7),0.0);
float eBS = eyeBrightnessSmooth.y/240.0;

#ifdef WorldTimeAnimation
float frametime = float(worldTime)/20.0*AnimationSpeed;
#else
float frametime = frameTimeCounter*AnimationSpeed;
#endif

float nolight = 1.0-(clamp((time-12000.0)/500.0,0.0,1.0)-clamp((time-13500.0)/500.0,0.0,1.0) + clamp((time-22500.0)/500.0,0.0,1.0)-clamp((time-23500.0)/500.0,0.0,1.0));

float luma(vec3 color){
	return dot(color,vec3(0.299, 0.587, 0.114));
}

vec3 calcLightColor(vec3 morning, vec3 day, vec3 night, vec3 rain){
	vec3 c = mix(night,mix(morning,day,pow(tB,0.75)),sunVisibility);
	return mix(c,length(c)*rain,rainStrength);
}

float Blinn_Phong(vec3 ppos, vec3 lightDir, vec3 surfaceNormal, float gloss, float visibility)  {
	vec3 viewDirection = normalize(-ppos);
	vec3 halfAngle = normalize(lightDir + viewDirection);
	float blinnTerm = dot(surfaceNormal, halfAngle);
	float pi = 3.1415927;
	float n =  pow(2.0,gloss);
	return pow(blinnTerm, n )*gloss*visibility;
}

float waterH(vec3 pos) {
	float noise = 0;
	noise+= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.1).r*1.0;
	noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.5).r*0.8;
	noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 2.5).r*0.6;
	noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 5.0).r*0.4;
	noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 8.0).r*0.2;

	noise /= pow(max(length(pos),4.0),0.35);
	return noise;
}

vec2 getRefract(vec2 coord, vec3 posxz){
	float deltaPos = 0.1;
	float h0 = waterH(posxz);
	float h1 = waterH(posxz + vec3(deltaPos,0.0,0.0));
	float h2 = waterH(posxz + vec3(-deltaPos,0.0,0.0));
	float h3 = waterH(posxz + vec3(0.0,0.0,deltaPos));
	float h4 = waterH(posxz + vec3(0.0,0.0,-deltaPos));

	float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
	float yDelta = ((h3-h0)+(h0-h4))/deltaPos;

	vec2 newcoord = coord + vec2(xDelta,yDelta)*0.01;
	float water = texture2D(colortex5, newcoord).g;
	water = float(water > 0.08 && water < 0.12);
	float limit = max(1.0-water,float(newcoord.x < 0.0 || newcoord.x > 1.0 || newcoord.y < 0.0 || newcoord.y > 1.0));

	return mix(newcoord,coord,limit);
}

float ld(float depth) {
   return (2.0 * near) / (far + near - depth * (far - near));
}

#if LightShaft == 1
vec2 getGRCoord(vec2 coord, vec2 dcoord, float str, float dither){
return coord - dcoord*dither*str;
}
#endif

#include "lib/fragPos.glsl"
#include "lib/shadowPos.glsl"
#ifdef RPSupport
#include "lib/shadows.glsl"
#endif
#include "lib/lightColors.glsl"
#include "lib/torchColor.glsl"
#include "lib/waterColor.glsl"
#include "lib/fogCalc.glsl"
#include "lib/dither.glsl"
#include "lib/ambientOcclusion.glsl"

#if LightShaft == 2
float distx(float dist){
	return (far * (dist - near)) / (dist * (far - near));
}

float getDepth(float depth) {
    return 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));
}

vec4 getShadowSpace(float shadowdepth, vec2 texcoord){
	vec4 fragpos = getFragPos(texcoord.xy,shadowdepth);

	vec4 wpos = getWorldPos(fragpos);
	wpos = getShadowPos(wpos);
	
	float distb = sqrt(wpos.x * wpos.x + wpos.y * wpos.y);
	float distortFactor = 1.0 - shadowMapBias + distb * shadowMapBias;
	wpos = distortShadow(wpos,distortFactor);
	
	return wpos;
}

float getnoise(vec2 pos){
	return fract(sin(dot(pos, vec2(12.9898, 4.1414))) * 43758.5453);
}

float hnoise(vec2 pos){
	vec2 flr = floor(pos);
	vec2 frc = fract(pos);
	frc = frc*frc*(3-2*frc);
	
	float noisedl = getnoise(flr);
	float noisedr = getnoise(flr+vec2(1.0,0.0));
	float noiseul = getnoise(flr+vec2(0.0,1.0));
	float noiseur = getnoise(flr+vec2(1.0,1.0));
	float noise= mix(mix(noisedl,noisedr,frc.x),mix(noiseul,noiseur,frc.x),frc.y);
	return noise;
}

float vcnoise(vec3 pos){
	vec3 flr = floor(pos);
	vec3 frc = fract(pos);
	float yadd = 32.0;
	frc = frc*frc*(3.0-2.0*frc);
	
	/*
	float noise = getnoise(flr.xz+flr.y*yadd);
	return noise
	*/
	
	float noisebdl = getnoise(flr.xz+flr.y*yadd);
	float noisebdr = getnoise(flr.xz+flr.y*yadd+vec2(1.0,0.0));
	float noisebul = getnoise(flr.xz+flr.y*yadd+vec2(0.0,1.0));
	float noisebur = getnoise(flr.xz+flr.y*yadd+vec2(1.0,1.0));
	float noisetdl = getnoise(flr.xz+flr.y*yadd+yadd);
	float noisetdr = getnoise(flr.xz+flr.y*yadd+yadd+vec2(1.0,0.0));
	float noisetul = getnoise(flr.xz+flr.y*yadd+yadd+vec2(0.0,1.0));
	float noisetur = getnoise(flr.xz+flr.y*yadd+yadd+vec2(1.0,1.0));
	float noise= mix(mix(mix(noisebdl,noisebdr,frc.x),mix(noisebul,noisebur,frc.x),frc.z),mix(mix(noisetdl,noisetdr,frc.x),mix(noisetul,noisetur,frc.x),frc.z),frc.y);
	return noise;
}

vec3 getVolumetricRays(float pixeldepth, vec3 color, float alpha, float water, float dither) {
	vec3 vl = vec3(0.0);
	float slength = 2.25;
	//float slength = 2.0+14.0*eBS;
	vec3 watercol = water_c*sqrt(cmult)/sqrt(water_a);

	float maxDist = 128.0;
	float minDist = 0.05;
	vec4 worldposition = vec4(0.0);
	
	float pixeldepthb = texture2D(depthtex0,texcoord.xy).r;
	float w = 0.0;
	
	for (minDist; minDist < maxDist; ) {
		minDist += dither*slength;
		if (getDepth(pixeldepth) < minDist || (getDepth(pixeldepthb) < minDist && alpha > 0.99)){
			break;
		}
		worldposition = getShadowSpace(distx(minDist),texcoord.st);
		if (length(worldposition.xy*2.0-1.0)<1.0){
			vec3 sample = vec3(shadow2D(shadowtex0, vec3(worldposition.rg, worldposition.b)).z);
			vec3 colsample = vec3(0.0);
			#ifdef ShadowColor
			float testsample = shadow2D(shadowtex1, vec3(worldposition.rg, worldposition.b)).z;
			if (testsample-sample.r > 0.9) colsample = shadow2D(shadowcolor, vec3(worldposition.rg, worldposition.b)).rgb;
			#endif
			sample = pow(max(sample,colsample),vec3(2.2));
			if (getDepth(pixeldepthb) < minDist) sample *= mix(mix(vec3(1.0),watercol,water*(1.0-isEyeInWater)),color,pow(alpha,1.0/2.2));
			else sample *= mix(vec3(1.0),watercol,isEyeInWater);
			//Volumetric fog
			/*
			vec4 worldpos = getWorldPos(getFragPos(texcoord.st,distx(minDist)))+vec4(cameraPosition.xyz,0.0);
			worldpos.xyz += vec3(32.0,4.0,32.0)*(vec3(vcnoise(worldpos.xyz*0.01+vec3(frametime*0.025,0.0,0.0)),vcnoise(worldpos.xyz*0.01+vec3(0.0,frametime*0.025,0.0)),vcnoise(worldpos.xyz*0.01+vec3(0.0,0.0,frametime*0.025)))-0.5);
			sample *= 0.5*(vcnoise(worldpos.xyz*0.25)+vcnoise(worldpos.xyz*1.0)+vcnoise(worldpos.xyz*4.0)+vcnoise(worldpos.xyz*16.0));
			*/
			vl += sample*minDist/(0.25+minDist);
		}
		else{
			vl += 1.0;
		}
		minDist += slength*(1.0-dither);
		slength *= 1.7;
		w += 1.0;
	}
	vl /= 8.0+w;
	//vl /= 16.0;
	
	return vec3(vl);
}
#endif

void main() {
	
	
	vec3 color = pow(texture2D(colortex0, texcoord.st).rgb,vec3(2.2))*MAX_COLOR_RANGE;
	vec3 colorb = vec3(0.0);
	vec3 aux = texture2D(colortex5, texcoord.st).rgb;
	float ao = 1.0;
	float spec = texture2D(colortex0, texcoord.st).a;
	float specemissive = 0.0;
	vec3 gr = vec3(0.0);
	
	float dither = bayer64(gl_FragCoord.xy);
	
	#ifdef LightmapBanding
	aux = vec3(floor(aux.r*16.0)/16.0,aux.g,floor(aux.b*16.0)/16.0);
	#endif
	
	vec3 light = calcLightColor(light_m, light_d, light_n, light_r);
	vec3 ambient = calcLightColor(sky_m, sky_d, sky_n, sky_r);
	
	float skymap = pow(aux.r,2.2);
	float matflag = aux.g;
	float torchmap = pow(aux.b,2.2)*0.2 + pow(max(aux.b-0.725,0.0)*5.0,4.0);
	float alpha = texture2D(colortex1,texcoord.xy).a;
	
	float translucent = float(matflag > 0.05);
	float water = float(matflag > 0.08 && matflag < 0.12);
	float particle = float(matflag > 0.28 && matflag < 0.32);
	float hand = float(matflag > 0.38 && matflag < 0.42);	
	
	float pixeldepth = texture2D(depthtex0,texcoord.xy).x;
	
	vec4 fragpos = getFragPos(texcoord.xy,pixeldepth+0.37*hand);
	
	if (translucent > 0.9 && alpha > 0.0001) {
		colorb = max(texture2D(colortex1, texcoord.st).rgb,vec3(1.0/pow(255.0,2.2)))*MAX_COLORB_RANGE;
		vec3 normal = texture2D(colortex3, texcoord.st).rgb*2.0-1.0;
		
		vec4 worldpos = getWorldPos(fragpos);
		
		#ifdef WaterRefraction
		if (water > 0.9 && isEyeInWater < 0.9){
			vec2 newcoord = getRefract(texcoord.xy,worldpos.xyz+cameraPosition.xyz);
			color = pow(texture2D(colortex0, newcoord).rgb,vec3(2.2))*MAX_COLOR_RANGE;
			colorb = max(texture2D(colortex1, newcoord.st).rgb,vec3(1.0/pow(255.0,2.2)))*MAX_COLORB_RANGE;
		}
		#endif
		
		color *= mix(vec3(1.0),sqrt(colorb),alpha*alpha);
		
		float absorb = float((water > 0.9 && isEyeInWater < 0.9) || (water < 0.9 && isEyeInWater > 0.9));
		vec3 uw_tint = mix(vec3(1.0),pow(water_c,vec3((1.0-skymap)*2.0))*skymap,absorb*max(1.0-pow(aux.b,2.2),0.0));
		
		#ifdef AO
		ao = dbao(depthtex0, dither);
		#endif
		
		colorb *= ao*uw_tint;
		
		#ifdef WaterV
		if(water > 0.9){
			color*= (1.0+colorb*water_a*4.0);
			alpha*= sqrt(water_a);
		}
		#endif
		
		color = mix(color,colorb,alpha);
		
		#ifdef Fog
		if (isEyeInWater < 0.9 && particle < 0.9) color = calcFog(color, light, fragpos.xyz);
		#endif
		
		#ifdef RPSupport
		float NdotL = max(dot(normal, lightVector)*1.05-0.05,0.0);
		float shading = getShadows(shadowtex0, worldpos, NdotL, 0.0, dither);
		shading *= max(NdotL*nolight,0.0);
		
		float smoothness = texture2D(colortex6,texcoord.xy).r;
		if (water < 0.9){
			spec = Blinn_Phong(fragpos.xyz,lightVector,normal,2.0+(10.0+moonVisibility)*smoothness,shading);
			spec = spec/(1.0+spec);
			}
		#endif
	}
	
	#if LightShaft == 1
	{
		vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
		tpos = vec4(tpos.xyz/tpos.w,1.0);
		tpos.xy = tpos.xy/tpos.z;
		vec2 lightPos = tpos.xy*0.5+0.5;
		float truepos = sunPosition.z/abs(sunPosition.z);
		
		vec2 deltacoord = texcoord.xy-lightPos;
		vec2 grcoord = texcoord.xy + deltacoord*0.1;
		
		float grfade = pow(max(1.0-length(deltacoord*vec2(1.0,1.0/aspectRatio))*2.0,0.0),2.2);
		float centerdist = 0.25 + 0.75*max(1.0-length(tpos.xy*vec2(aspectRatio,1.0)*0.5),0.0);
		float grw = 0.0;
		
		if (grfade > 0.001 && (truepos < 0.0 && sunVisibility > 0.001 || truepos > 0.0 && moonVisibility > 0.001)){
			for (int i = 0; i < 3; i++) {
			float samplea = texture2D(colortex4,getGRCoord(grcoord,deltacoord,0.3,dither)).g;
			vec4 sampleb = texture2D(colortex1,getGRCoord(grcoord,deltacoord,0.3,dither));
			if(isEyeInWater > 0.9){
				float samplec = texture2D(colortex5,getGRCoord(grcoord,deltacoord,0.3,dither)).g;
				if (samplec < 0.12){
					sampleb.rgb = water_c;
					sampleb.a = water_a;
				}
			}
			sampleb.rgb = max(normalize(sampleb.rgb*MAX_COLOR_RANGE),vec3(0.0));
			gr += float(samplea < 0.02)*mix(vec3(1.0),vec3(sampleb.rgb),pow(sampleb.a,0.2))*(1.0-sampleb.a)*centerdist;
			grcoord -= deltacoord*0.3;
			}
		}
		
		gr /= 3.0;
	}
	#endif
	
	#if LightShaft == 2
	gr = getVolumetricRays(texture2D(depthtex1,texcoord.xy).x,sqrt(colorb),alpha,water,dither)*nolight;
	#endif
	
/* DRAWBUFFERS:01 */
	
	gl_FragData[0] = vec4(pow(color/MAX_COLOR_RANGE,vec3(1.0/2.2)),spec);
	gl_FragData[1] = vec4(gr,0.0);

}
